/*
 * Copyright: (c) 2004-2009 Mayo Foundation for Medical Education and 
 * Research (MFMER). All rights reserved. MAYO, MAYO CLINIC, and the
 * triple-shield Mayo logo are trademarks and service marks of MFMER.
 *
 * Except as contained in the copyright notice above, or as used to identify 
 * MFMER as the author of this software, the trade names, trademarks, service
 * marks, or product names of the copyright holder shall not be used in
 * advertising, promotion or otherwise in connection with this software without
 * prior written authorization of the copyright holder.
 * 
 * Licensed under the Eclipse Public License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 * 
 * 		http://www.eclipse.org/legal/epl-v10.html
 * 
 */
package edu.mayo.informatics.lexgrid.convert.swt;

import java.io.StringWriter;

import org.eclipse.swt.custom.StyledText;

/**
 * A StringWriter implementation that writes to a SWT StyledText Composite. Used
 * by the log viewer.
 * 
 * @author <A HREF="mailto:armbrust.daniel@mayo.edu">Dan Armbrust</A>
 * @version subversion $Revision: 7198 $ checked in on $Date: 2008-02-15
 *          18:08:36 +0000 (Fri, 15 Feb 2008) $
 */
public class CustomStringWriter extends StringWriter {
    private StyledText area_;

    public CustomStringWriter(StyledText area) {
        super();
        area_ = area;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.io.Writer#write(char[], int, int)
     */
    public void write(char[] cbuf, int off, int len) {
        super.write(cbuf, off, len);
        update();
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.io.Writer#write(int)
     */
    public void write(int c) {
        super.write(c);
        update();
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.io.Writer#write(java.lang.String, int, int)
     */
    public void write(String str, int off, int len) {
        super.write(str, off, len);
        update();
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.io.Writer#write(java.lang.String)
     */
    public void write(String str) {
        super.write(str);
        update();
    }

    private void update() {
        area_.getDisplay().syncExec(new Runnable() {
            public void run() {
                area_.append(CustomStringWriter.this.getBuffer().substring(area_.getText().length()));
                area_.setSelection(area_.getText().length());
            }
        });
    }
}