/*
 * Copyright: (c) 2004-2009 Mayo Foundation for Medical Education and 
 * Research (MFMER). All rights reserved. MAYO, MAYO CLINIC, and the
 * triple-shield Mayo logo are trademarks and service marks of MFMER.
 *
 * Except as contained in the copyright notice above, or as used to identify 
 * MFMER as the author of this software, the trade names, trademarks, service
 * marks, or product names of the copyright holder shall not be used in
 * advertising, promotion or otherwise in connection with this software without
 * prior written authorization of the copyright holder.
 * 
 * Licensed under the Eclipse Public License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 * 
 * 		http://www.eclipse.org/legal/epl-v10.html
 * 
 */
package org.LexGrid.LexBIG.Impl.codedNodeSetOperations;

import org.LexGrid.LexBIG.Exceptions.LBParameterException;
import org.LexGrid.LexBIG.Impl.codedNodeSetOperations.interfaces.Operation;
import org.LexGrid.LexBIG.Impl.codedNodeSetOperations.interfaces.Restriction;
import org.LexGrid.LexBIG.LexBIGService.CodedNodeSet.ActiveOption;
import org.LexGrid.annotations.LgClientSideSafe;

/**
 * Holder for the RestrictToStatus operation.
 * 
 * @author <A HREF="mailto:armbrust.daniel@mayo.edu">Dan Armbrust</A>
 * @author <A HREF="mailto:erdmann.jesse@mayo.edu">Jesse Erdmann</A>
 * @version subversion $Revision: $ checked in on $Date: $
 */
public class RestrictToStatus implements Restriction, Operation {

    private static final long serialVersionUID = 4030470756164999491L;
    private ActiveOption activeOption_;
    private String[] status_;

    public RestrictToStatus(ActiveOption activeOption, String[] status) throws LBParameterException {
        if (activeOption == null && (status == null || status.length == 0)) {
            throw new LBParameterException("You must provide at least one parameter", "activeOption or status");
        }
        this.activeOption_ = activeOption;
        this.status_ = status;
    }

    /**
     * @return the activeOption_
     */
    @LgClientSideSafe
    public ActiveOption getActiveOption() {
        return this.activeOption_;
    }

    /**
     * @return the status_
     */
    @LgClientSideSafe
    public String[] getStatus() {
        return this.status_;
    }
}