/*
 * Copyright: (c) 2004-2009 Mayo Foundation for Medical Education and 
 * Research (MFMER). All rights reserved. MAYO, MAYO CLINIC, and the
 * triple-shield Mayo logo are trademarks and service marks of MFMER.
 *
 * Except as contained in the copyright notice above, or as used to identify 
 * MFMER as the author of this software, the trade names, trademarks, service
 * marks, or product names of the copyright holder shall not be used in
 * advertising, promotion or otherwise in connection with this software without
 * prior written authorization of the copyright holder.
 * 
 * Licensed under the Eclipse Public License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 * 
 * 		http://www.eclipse.org/legal/epl-v10.html
 * 
 */
package edu.mayo.informatics.resourcereader.obo;

import java.util.Vector;

import org.LexGrid.messaging.CachingMessageDirectorIF;

import edu.mayo.informatics.resourcereader.core.StringUtils;
import edu.mayo.informatics.resourcereader.core.IF.ResourceEntity;

/**
 * A commom class that is used to store common entity information
 * 
 * @author <A HREF="mailto:sharma.deepak2@mayo.edu">Deepak Sharma</A>
 * @version subversion $Revision: $ checked in on $Date: $
 */
public abstract class OBOEntity extends OBO implements ResourceEntity {
    public String id = null;
    public String prefix = null;
    public String name = null;
    public String namespace = null;
    public boolean isAnonymous = false;
    public boolean isObsolete = false;
    public String replacedBy = null;
    public String consider = null;
    public String comment = null;
    public String definition = null;
    public Vector<OBODbxref> definitionSources = new Vector<OBODbxref>();
    public Vector<OBODbxref> dbXrefs = new Vector<OBODbxref>();
    public Vector<OBOSynonym> synonyms = new Vector<OBOSynonym>();
    public Vector<String> altIds = new Vector<String>();

    public OBOEntity() {
        super();
    }

    public OBOEntity(CachingMessageDirectorIF rLogger) {
        super(rLogger);
    }

    public Vector<String> getAltIds() {
        return altIds;
    }

    public void setAltIds(Vector<String> altIds) {
        this.altIds = altIds;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public String getConsider() {
        return consider;
    }

    public void setConsider(String consider) {
        this.consider = consider;
    }

    public Vector<OBODbxref> getDbXrefs() {
        return dbXrefs;
    }

    public void setDbXrefs(Vector<OBODbxref> dbXrefs) {
        this.dbXrefs = dbXrefs;
    }

    public String getDefinition() {
        return definition;
    }

    public void setDefinition(String definition) {
        this.definition = definition;
    }

    public Vector<OBODbxref> getDefinitionSources() {
        return definitionSources;
    }

    public void setDefinitionSources(Vector<OBODbxref> definitionSources) {
        this.definitionSources = definitionSources;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public boolean isAnonymous() {
        return isAnonymous;
    }

    public void setAnonymous(boolean isAnonymous) {
        this.isAnonymous = isAnonymous;
    }

    public boolean isObsolete() {
        return isObsolete;
    }

    public void setObsolete(boolean isObsolete) {
        this.isObsolete = isObsolete;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getNamespace() {
        return namespace;
    }

    public void setNamespace(String namespace) {
        this.namespace = namespace;
    }

    public String getPrefix() {
        return prefix;
    }

    public void setPrefix(String prefix) {
        this.prefix = prefix;
    }

    public String getReplacedBy() {
        return replacedBy;
    }

    public void setReplacedBy(String replacedBy) {
        this.replacedBy = replacedBy;
    }

    public Vector<OBOSynonym> getSynonyms() {
        return synonyms;
    }

    public void setSynonyms(Vector<OBOSynonym> synonyms) {
        this.synonyms = synonyms;
    }

    /**
     * The merge operation would cause all the non null string data elements of
     * entity to overwrite their counterparts in the current instance. The
     * Vector data elements are merged. The boolean variables are overwritten
     * with the data counterpart in entity.
     * 
     * @param entity
     */
    public void merge(OBOEntity entity) {
        if (!StringUtils.isNull(entity.id)) {
            this.id = entity.id;
        }
        if (!StringUtils.isNull(entity.prefix)) {
            this.prefix = entity.prefix;
        }
        if (!StringUtils.isNull(entity.name)) {
            this.name = entity.name;
        }
        if (!StringUtils.isNull(entity.namespace)) {
            this.namespace = entity.namespace;
        }
        if (!StringUtils.isNull(entity.replacedBy)) {
            this.replacedBy = entity.replacedBy;
        }
        if (!StringUtils.isNull(entity.consider)) {
            this.consider = entity.consider;
        }
        if (!StringUtils.isNull(entity.comment)) {
            this.comment = entity.comment;
        }
        if (!StringUtils.isNull(entity.definition)) {
            this.definition = entity.definition;
        }

        this.isAnonymous = entity.isAnonymous;
        this.isObsolete = entity.isObsolete;
        this.definitionSources.addAll(entity.definitionSources);
        this.dbXrefs.addAll(entity.dbXrefs);
        this.synonyms.addAll(entity.synonyms);
        this.altIds.addAll(entity.altIds);

    }
}