/*
 * Copyright: (c) 2004-2009 Mayo Foundation for Medical Education and 
 * Research (MFMER). All rights reserved. MAYO, MAYO CLINIC, and the
 * triple-shield Mayo logo are trademarks and service marks of MFMER.
 *
 * Except as contained in the copyright notice above, or as used to identify 
 * MFMER as the author of this software, the trade names, trademarks, service
 * marks, or product names of the copyright holder shall not be used in
 * advertising, promotion or otherwise in connection with this software without
 * prior written authorization of the copyright holder.
 * 
 * Licensed under the Eclipse Public License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 * 
 * 		http://www.eclipse.org/legal/epl-v10.html
 * 
 */
package org.lexgrid.loader.processor;

import org.LexGrid.persistence.model.CodingSchemeSupportedAttrib;
import org.LexGrid.persistence.model.CodingSchemeSupportedAttribId;
import org.lexgrid.loader.data.codingScheme.CodingSchemeNameSetter;
import org.lexgrid.loader.processor.support.SupportedAttribResolver;
import org.springframework.batch.item.ItemProcessor;

/**
 * The Class CodingSchemeSupportedAttribProcessor.
 * 
 * @author <a href="mailto:kevin.peterson@mayo.edu">Kevin Peterson</a>
 */
public class CodingSchemeSupportedAttribProcessor<I> extends CodingSchemeNameAwareProcessor
	implements ItemProcessor<I,CodingSchemeSupportedAttrib>{

	/** The supported attrib resolver. */
	private SupportedAttribResolver<I> supportedAttribResolver;
	
	/* (non-Javadoc)
	 * @see org.springframework.batch.item.ItemProcessor#process(java.lang.Object)
	 */
	public CodingSchemeSupportedAttrib process(I item) throws Exception {
		CodingSchemeSupportedAttrib supportedAttrib = new CodingSchemeSupportedAttrib();
		CodingSchemeSupportedAttribId supportedAttribId = new CodingSchemeSupportedAttribId();
		supportedAttribId.setCodingSchemeName(getCodingSchemeNameSetter().getCodingSchemeName());
		supportedAttribId.setSupportedAttributeTag(supportedAttribResolver.getSupportedAttributeTag());
		supportedAttribId.setId(supportedAttribResolver.getId(item));
		supportedAttribId.setIdValue(supportedAttribResolver.getIdVal(item));
		supportedAttribId.setVal1(supportedAttribResolver.getVal1(item));
		supportedAttrib.setUri(supportedAttribResolver.getUri(item));
		supportedAttrib.setVal2(supportedAttribResolver.getVal2(item));
		supportedAttrib.setId(supportedAttribId);
		return supportedAttrib;
	}

	/**
	 * Gets the supported attrib resolver.
	 * 
	 * @return the supported attrib resolver
	 */
	public SupportedAttribResolver<I> getSupportedAttribResolver() {
		return supportedAttribResolver;
	}

	/**
	 * Sets the supported attrib resolver.
	 * 
	 * @param supportedAttribResolver the new supported attrib resolver
	 */
	public void setSupportedAttribResolver(
			SupportedAttribResolver<I> supportedAttribResolver) {
		this.supportedAttribResolver = supportedAttribResolver;
	}
}
