/*
 * Copyright: (c) 2004-2009 Mayo Foundation for Medical Education and 
 * Research (MFMER). All rights reserved. MAYO, MAYO CLINIC, and the
 * triple-shield Mayo logo are trademarks and service marks of MFMER.
 *
 * Except as contained in the copyright notice above, or as used to identify 
 * MFMER as the author of this software, the trade names, trademarks, service
 * marks, or product names of the copyright holder shall not be used in
 * advertising, promotion or otherwise in connection with this software without
 * prior written authorization of the copyright holder.
 * 
 * Licensed under the Eclipse Public License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 * 
 * 		http://www.eclipse.org/legal/epl-v10.html
 * 
 */
package org.lexgrid.loader.meta.integration.dataload;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import org.LexGrid.LexBIG.DataModel.Core.ResolvedConceptReference;
import org.LexGrid.LexBIG.Utility.Constructors;
import org.LexGrid.commonTypes.Property;
import org.junit.Before;
import org.junit.Test;
import org.lexgrid.loader.rrf.constants.RrfLoaderConstants;

import test.util.DataTestUtils;
import util.integration.LoadMetaForIntegration;

public class MrhierPropertyQualifierTestIT extends DataLoadTestBase {
	
	private ResolvedConceptReference refC0000005;
	private ResolvedConceptReference refC0036775;
	private ResolvedConceptReference refC0001555;
	private ResolvedConceptReference refCL385598;
	
	@Before
	public void buildTestEntity() throws Exception {
		refC0000005 = cng.resolveAsList(Constructors.createConceptReference("C0000005", LoadMetaForIntegration.META_URN), true, false, 0, 1, null, null, null, -1).getResolvedConceptReference()[0];	
		refC0036775 = cng.resolveAsList(Constructors.createConceptReference("C0036775", LoadMetaForIntegration.META_URN), true, false, 0, 1, null, null, null, -1).getResolvedConceptReference()[0];
		refC0001555 = cng.resolveAsList(Constructors.createConceptReference("C0001555", LoadMetaForIntegration.META_URN), true, false, 0, 1, null, null, null, -1).getResolvedConceptReference()[0];
		refCL385598 = cng.resolveAsList(Constructors.createConceptReference("CL385598", LoadMetaForIntegration.META_URN), true, false, 0, 1, null, null, null, -1).getResolvedConceptReference()[0];
	}
	
	@Test
	public void testNotNull() throws Exception {	
		assertNotNull(refC0000005);
		assertNotNull(refC0036775);
		assertNotNull(refC0001555);
		assertNotNull(refCL385598);
	}
	
	@Test
	public void testIsC0000005HcdPropQualifierPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithValue(refC0000005.getEntity(), "(131)I-Macroaggregated Albumin");
		
		assertTrue(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.HCD_QUALIFIER, "testHierarchy", prop)				
		);
	}
	
	@Test
	public void testIsNotHierarchyParticipatingC0000005HcdPropQualifierNotPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithValue(refC0000005.getEntity(), "(131)I-MAA");
		
		assertFalse(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.HCD_QUALIFIER, "testHierarchy", prop)				
		);
	}
	
	@Test
	public void testIsC0036775HcdPropQualifierPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithValue(refC0036775.getEntity(), "Serum Albumin, Radio-Iodinated");
		
		assertTrue(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.HCD_QUALIFIER, "testHierarchy", prop)				
		);
	}
	
	@Test
	public void testIsNotHierarchyParticipatingC0036775HcdPropQualifierNotPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithValue(refC0036775.getEntity(), "Serum Albumin, Radio Iodinated");
		
		assertFalse(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.HCD_QUALIFIER, "testHierarchy", prop)				
		);
	}
	
	@Test
	public void testIsC0001555HcdPropQualifierPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithId(refC0001555.getEntity().getPresentation(), "A5703273");
		
		assertTrue(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.HCD_QUALIFIER, "testHierarchy", prop)				
		);
	}
	
	@Test
	public void testIsNotHierarchyParticipatingC0001555HcdPropQualifierNotPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithId(refC0001555.getEntity().getPresentation(), "A7737265");
		
		assertFalse(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.HCD_QUALIFIER, "testHierarchy", prop)				
		);
	}
	
	@Test
	public void testIsCL385598HcdPropQualifierPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithId(refCL385598.getEntity().getPresentation(), "A5784826");
		
		assertTrue(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.HCD_QUALIFIER, "testHierarchy", prop)				
		);
	}
	
	@Test
	public void testIsNotHierarchyParticipatingCL385598HcdPropQualifierNotPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithId(refCL385598.getEntity().getPresentation(), "A7709649");
		
		assertFalse(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.HCD_QUALIFIER, "testHierarchy", prop)				
		);
	}
	
	@Test
	public void testIsC0000005PtrPropQualifierPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithValue(refC0000005.getEntity(), "(131)I-Macroaggregated Albumin");
		
		assertTrue(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.PTR_QUALIFIER, "A3586555.A5703273.A5784826", prop)				
		);
	}
	
	@Test
	public void testIsNotHierarchyParticipatingC0000005PtrPropQualifierNotPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithValue(refC0000005.getEntity(), "(131)I-MAA");
		
		assertFalse(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.PTR_QUALIFIER, "A3586555.A5703273.A5784826", prop)				
		);
	}
	
	@Test
	public void testIsC0036775PtrPropQualifierPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithValue(refC0036775.getEntity(), "Serum Albumin, Radio-Iodinated");
		
		assertTrue(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.PTR_QUALIFIER, "A3586555.A5703273.A5784826", prop)			
		);
	}
	
	@Test
	public void testIsNotHierarchyParticipatingC0036775PtrPropQualifierNotPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithValue(refC0036775.getEntity(), "Serum Albumin, Radio Iodinated");
		
		assertFalse(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.PTR_QUALIFIER, "A3586555.A5703273.A5784826", prop)			
		);
	}
	
	@Test
	public void testIsC0001555PtrPropQualifierPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithId(refC0001555.getEntity().getPresentation(), "A5703273");
		
		assertTrue(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.PTR_QUALIFIER, "A3586555.A5703273.A5784826", prop)			
		);
	}
	
	@Test
	public void testIsNotHierarchyParticipatingC0001555PtrPropQualifierNotPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithId(refC0001555.getEntity().getPresentation(), "A7737265");
		
		assertFalse(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.PTR_QUALIFIER, "A3586555.A5703273.A5784826", prop)				
		);
	}
	
	@Test
	public void testIsCL385598PtrPropQualifierPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithId(refCL385598.getEntity().getPresentation(), "A5784826");
		
		assertTrue(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.PTR_QUALIFIER, "A3586555.A5703273.A5784826", prop)				
		);
	}
	
	@Test
	public void testIsNotHierarchyParticipatingCL385598PtrPropQualifierNotPresent() throws Exception {	
		Property prop = DataTestUtils.getPropertyWithId(refCL385598.getEntity().getPresentation(), "A7709649");
		
		assertFalse(
				DataTestUtils.isQualifierNameAndValuePresentInProperty(
						RrfLoaderConstants.PTR_QUALIFIER, "A3586555.A5703273.A5784826", prop)				
		);
	}
}